更新时间：2022-06-14

# 子区消息管理

子区消息消息类型属于群聊消息类型，与普通群组消息的区别是需要添加 `isChatThread` 标记。本文介绍 即时通讯 im_flutter_sdk 如何发送、接收以及撤回子区消息。

## 技术原理

即时通讯 im_flutter_sdk 提供 `EMChatManager`、`EMMessage` 和 `EMChatThread` 类，用于管理子区消息，支持你通过调用 API 在项目中实现如下功能：

- 发送子区消息
- 接收子区消息
- 撤回子区消息
- 获取子区消息

下图展示在客户端发送和接收消息的工作流程：

![img](https://docs-im.easemob.com/_detail/ccim/web/sendandreceivemsg.png?id=ccim%3Aandroid%3Amessage2)

如上图所示，消息收发流程如下：

1. 用户 A 发送一条消息到消息服务器；
2. 对于子区消息，服务器投递给子区内其他每一个成员；
3. 用户收到消息。

## 前提条件

开始前，请确保满足以下条件：

- 已集成 IM `(3.9.3.1 以上版本)` 的基本功能，账户登录成功。
- 完成 SDK 初始化，详见 [快速开始](https://docs-im.easemob.com/ccim/flutter/quickstart)。
- 了解即时通讯 IM 的使用限制，详见 [使用限制](https://docs-im.easemob.com/ccim/limitation)。
- 联系商务开通子区功能。

## 实现方法

本节介绍如何使用即时通讯 im_flutter_sdk 提供的 API 实现上述功能。

### 发送子区消息

发送子区消息和发送群组消息的方法基本一致，详情请参考 [发送消息](https://docs-im.easemob.com/ccim/rn/message2#发送文本消息)。唯一不同的是，发送子区消息需要指定标记 `isChatThreadMessage` 为 `true`。

示例代码如下：

```dart
// targetGroup 为群组id
EMMessage msg = EMMessage.createTxtSendMessage(
  targetId: targetGroup,
  content: content,
);
// 设置为群组消息
msg.chatType = ChatType.GroupChat;
// isChatThreadMessage: 是否是子区消息，这里设置为 `true`，即是子区消息
msg.isChatThreadMessage = true;
EMClient.getInstance.chatManager.sendMessage(msg);
```

### 接收子区消息

接收消息的具体逻辑，请参考[接收消息](https://docs-im.easemob.com/ccim/flutter/message2#接收消息)，此处只介绍子区消息和其他消息的区别。

子区有新增消息时，子区所属群组的所有成员收到 `EMChatThreadManagerListener#onChatThreadUpdated` 回调，子区成员收到 `EMChatManagerListener#onMessagesReceived` 回调。

示例代码如下：

```dart
class _ChatPageState extends State<ChatPage>
    implements EMChatThreadManagerListener, EMChatManagerListener {
  @override
  void initState() {
    super.initState();
    // 添加消息监听
    EMClient.getInstance.chatManager.addChatManagerListener(this);
    // 添加子区监听
    EMClient.getInstance.chatThreadManager.addChatThreadManagerListener(this);
  }

  @override
  void dispose() {

    // 移除消息监听
    EMClient.getInstance.chatManager.removeChatManagerListener(this);
    // 移除子区监听
    EMClient.getInstance.chatThreadManager.removeChatThreadManagerListener(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container();
  }

  // 子区更新回调
  @override
  void onChatThreadUpdate(EMChatThreadEvent event) {}

  // 收消息回调
  @override
  void onMessagesReceived(List<EMMessage> messages) {}

  // 其他回调接收省略，实际开发需添加
  ...
}

```

### 撤回子区消息

接收消息的具体逻辑，请参考[撤回消息](https://docs-im.easemob.com/ccim/flutter/message2#撤回消息)，此处只介绍子区消息和其他消息的区别。

子区有消息撤回时，子区所属群组的所有成员收到 `EMChatThreadManagerListener#onChatThreadUpdated` 回调，子区成员收到 `EMChatManagerListener#onMessagesRecalled` 回调。

示例代码如下：

```dart
```dart
class _ChatPageState extends State<ChatPage>
    implements EMChatThreadManagerListener, EMChatManagerListener {
  @override
  void initState() {
    super.initState();
    // 添加消息监听
    EMClient.getInstance.chatManager.addChatManagerListener(this);
    // 添加子区监听
    EMClient.getInstance.chatThreadManager.addChatThreadManagerListener(this);
  }

  @override
  void dispose() {

    // 移除消息监听
    EMClient.getInstance.chatManager.removeChatManagerListener(this);
    // 移除子区监听
    EMClient.getInstance.chatThreadManager.removeChatThreadManagerListener(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container();
  }

  // 子区更新回调
  @override
  void onChatThreadUpdate(EMChatThreadEvent event) {}

  // 消息撤回回调
  @override
  void onMessagesRecalled(List<EMMessage> messages) {}

  // 其他回调接收省略，实际开发需添加
  ...
}
```

### 从服务器获取子区消息 (消息漫游)

从服务器获取子区消息，请参考 [从服务器获取消息 (消息漫游)](https://docs-im.easemob.com/ccim/rn/message4#消息管理_从服务器获取消息_消息漫游)。
