# 管理群组

群组是支持多人沟通的即时通讯系统，本文指导你如何使用环信即时通讯 IM Flutter SDK 在实时互动 app 中创建并管理群组。

## 技术原理

环信即时通讯 IM Flutter SDK 提供 `EMGroup`、`EMGroupManager` 和 `EMGroupManagerListener` 类用于群组管理，支持你通过调用 API 在项目中实现如下功能：

- 创建、解散群组
- 加入、退出群组
- 获取群组详情
- 获取群成员列表
- 获取群组列表
- 屏蔽、解除屏蔽群消息
- 监听群组事件

## 前提条件

开始前，请确保满足以下条件：

- 完成 SDK 初始化，详见 [快速开始](https://docs-im.easemob.com/ccim/flutter/quickstart)；
- 了解环信即时通讯 IM API 的接口调用频率限制，详见 [使用限制](https://docs-im.easemob.com/ccim/limitation)；
- 了解群组和群成员的数量限制，详见 [套餐包详情](https://www.easemob.com/pricing/im)。

## 实现方法

本节介绍如何使用环信即时通讯 IM Flutter SDK 提供的 API 实现上述功能。

### 创建群组

在创建群组前，你需要设置群组类型 (`EMGroupStyle`) 和进群邀请是否需要对方同意 (`inviteNeedConfirm`)。

1. 群组类型 (`EMGroupStyle`) 的具体设置如下：

- `PrivateOnlyOwnerInvite` —— 私有群，只有群主和管理员可以邀请人进群；
- `PrivateMemberCanInvite` —— 私有群，所有群成员均可以邀请人进群；
- `PublicJoinNeedApproval` —— 公开群，群主和管理员可以邀请人进群，用户可以提交进群申请；
- `PublicOpenJoin` —— 公开群，任何人都可以进群，无需群主和群管理同意。

2. 进群邀请是否需要对方同意 (`inviteNeedConfirm`) 的具体设置如下：

- 进群邀请需要用户确认 (`EMGroupOptions#inviteNeedConfirm` 设置为 `true`)。创建群组并发出邀请后，根据受邀用户的 `EMOptions#autoAcceptGroupInvitation` 设置，处理逻辑如下：
  - 用户设置手动确认群组邀请 (`EMOptions#autoAcceptGroupInvitation` 设置为 `false`)。受邀用户收到 `EMGroupManagerListener#onInvitationReceivedFromGroup` 回调，并选择同意或拒绝入群邀请：
    - 用户同意入群邀请后，群主收到 `EMGroupManagerListener#onInvitationAcceptedFromGroup` 回调和 `EMGroupManagerListener#onMemberJoinedFromGroup` 回调，其他群成员收到 `EMGroupManagerListener#onMemberJoinedFromGroup` 回调；
    - 用户拒绝入群邀请后，群主收到 `EMGroupManagerListener#onInvitationDeclinedFromGroup` 回调。

- 进群邀请无需用户确认 (`EMGroupOptions.inviteNeedConfirm` 设置为 `false`)。创建群组并发出邀请后，无视用户的 `EMOptions#autoAcceptGroupInvitation` 设置，受邀用户直接进群。用户收到`EMGroupManagerListener#onAutoAcceptInvitationFromGroup` 回调，群主收到每个加入成员的 `EMGroupManagerListener#onInvitationAcceptedFromGroup` 回调和 `EMGroupManagerListener#onMemberJoinedFromGroup` 回调。

用户可以调用 `EMGroupManager#createGroup` 方法创建群组。

示例代码如下：

```dart
EMGroupOptions groupOptions = EMGroupOptions(
  style: EMGroupStyle.PrivateMemberCanInvite,
  inviteNeedConfirm: true,
);

String groupName = "newGroup";
String groupDesc = "group desc";
try {
  await EMClient.getInstance.groupManager.createGroup(
    groupName: groupName,
    desc: groupDesc,
    options: groupOptions,
  );
} on EMError catch (e) {
}
```

### 解散群组 <a name="destroy"></a>

仅群主可以调用 `DestroyGroup` 方法解散群组。群组解散时，其他群组成员收到 `OnDestroyedFromGroup` 回调并被踢出群组。

**注意：**
解散群组后，将删除本地数据库及内存中的群相关信息及群会话，谨慎操作。

示例代码如下：

```dart
SDKClient.Instance.GroupManager.DestroyGroup(groupId, new CallBack(
  onSuccess: () =>
  {
  },
  onError: (code, desc) =>
  {
  }
));
```

### 加入群组

根据 [创建群组](https://docs-preprod.agora.io/cn/agora-chat/agora_chat_group_flutter?platform=flutter#创建群组) 时的群组类型 (`GroupStyle`) 设置，加入群组的处理逻辑差别如下：

- 当群组类型为 `PublicOpenJoin` 时，用户可以直接加入群组，无需群主或群管理员同意，加入群组后，其他群成员收到 `EMGroupManagerListener#onMemberJoinedFromGroup` 回调；
- 当群组类型为 `PublicJoinNeedApproval` 时，用户可以申请进群，群主或群管理员收到 `EMGroupManagerListener#onRequestToJoinReceivedFromGroup` 回调，并选择同意或拒绝入群申请：
  - 群主或群管理员同意入群申请，申请人收到 `EMGroupManagerListener#onRequestToJoinAcceptedFromGroup` 回调，其他群成员收到`EMGroupManagerListener#onMemberJoinedFromGroup` 回调；
  - 群主或群管理员拒绝入群申请，申请人收到 `EMGroupManagerListener#onRequestToJoinDeclinedFromGroup` 回调。

**注意**
用户只能申请加入公开群组，私有群组不支持用户申请入群。

用户申请加入群组的步骤如下：

1. 调用 `EMGroupManager#fetchPublicGroupsFromServer` 方法从服务器获取公开群列表，查询到想要加入的群组 ID。

2. 调用 `EMGroupManager#joinPublicGroup` 方法传入群组 ID，申请加入对应群组。

示例代码如下：

```dart
// 获取公开群组列表
try {
  EMCursorResult<EMGroupInfo> result =
      await EMClient.getInstance.groupManager.fetchPublicGroupsFromServer();
} on EMError catch (e) {
}

// 申请加入群组
try {
  await EMClient.getInstance.groupManager.joinPublicGroup(groupId);
} on EMError catch (e) {
}
```

### 退出群组

群成员可以调用 `LeaveGroup` 方法退出群组，其他成员收到 `EMGroupManagerListener#onMemberExitedFromGroup` 回调。退出群组后，该用户将不再收到群消息。群主不能调用该接口退出群组，只能调用 [`DestroyGroup`](#destroy) 解散群组。

示例代码如下：

```dart
try {
  await EMClient.getInstance.groupManager.leaveGroup(groupId);
} on EMError catch (e) {
}
```

### 获取群组详情

群成员可以调用 `EMGroupManager#getGroupWithId` 方法从内存获取群组详情。返回结果包括：群组 ID、群组名称、群组描述、群组基本属性、群主、群组管理员列表，默认不包含群成员。

群成员也可以调用 `EMGroupManager#fetchGroupInfoFromServer` 方法从服务器获取群组详情。返回结果包括：群组 ID、群组名称、群组描述、群主、群组管理员列表以及群成员列表。

示例代码如下：

```dart
// 从本地获取群组
try {
  EMGroup? group = await EMClient.getInstance.groupManager.getGroupWithId(groupId);
} on EMError catch (e) {
}

// 从服务器获取群组详情
try {
  EMGroup group = await EMClient.getInstance.groupManager.fetchGroupInfoFromServer(groupId);
} on EMError catch (e) {
}
```

### 获取群成员列表

群成员可以调用 `EMGroupManager#fetchMemberListFromServer` 方法从服务器分页获取群成员列表。

示例代码如下：

```dart
try {
  EMCursorResult<String> result =
      await EMClient.getInstance.groupManager.fetchMemberListFromServer(
    groupId,
  );
} on EMError catch (e) {
}
```

### 获取群组列表

用户可以调用 `EMGroupManager#fetchJoinedGroupsFromServer` 方法从服务器获取自己加入和创建的群组列表。示例代码如下：

```dart
try {
  List<EMGroup> list =
      await EMClient.getInstance.groupManager.fetchJoinedGroupsFromServer();
} on EMError catch (e) {
}
```

用户可以调用 `EMGroupManager#getJoinedGroups` 方法加载本地群组列表。为了保证数据的正确性，需要先从服务器获取自己加入和创建的群组列表。示例代码如下：

```dart
try {
  List<EMGroup> list =
      await EMClient.getInstance.groupManager.getJoinedGroups();
} on EMError catch (e) {
}
```

用户还可以调用 `EMGroupManager#fetchPublicGroupsFromServer` 方法从服务器分页获取公开群组列表。示例代码如下：

```dart
try {
  EMCursorResult<EMGroupInfo> result =
      await EMClient.getInstance.groupManager.fetchPublicGroupsFromServer(
    pageSize: pageSize,
    cursor: cursor,
  );
} on EMError catch (e) {
}
```

### 屏蔽和解除屏蔽群消息

#### 屏蔽群消息

群成员可以调用 `EMGroupManager#blockGroup` 方法屏蔽群消息。屏蔽群消息后，该成员不再从指定群组接收群消息，群主和群管理员不能进行此操作。示例代码如下：

```dart
try {
  await EMClient.getInstance.groupManager.blockGroup(groupId);
} on EMError catch (e) {
}
```

#### 解除屏蔽群消息

群成员可以调用 `EMGroupManager#unblockGroup` 方法解除屏蔽群消息。示例代码如下：

```dart
try {
  await EMClient.getInstance.groupManager.unblockGroup(groupId);
} on EMError catch (e) {
}
```

#### 检查自己是否屏蔽群消息

群成员可以调用 `EMGroupManager#fetchGroupInfoFromServer` 方法并通过 `EMGroup#messageBlocked` 字段检查自己是否屏蔽了群消息。

示例代码如下：

```dart
try {
  EMGroup group = await EMClient.getInstance.groupManager
      .fetchGroupInfoFromServer(groupId);
  // 检查用户是否屏蔽了该群的群消息
  if (group.messageBlocked == true) {

  }
} on EMError catch (e) {
}
```

### 监听群组事件

`EMGroupManagerListener` 类中提供群组事件的监听接口。开发者可以通过设置此监听，获取群组中的事件，并做出相应处理。如果不再使用该监听，需要移除，防止出现内存泄漏。

示例代码如下：

```dart
class _GroupPageState extends State<GroupPage> implements EMGroupManagerListener {
  @override
  void initState() {
    // 注册群组
    EMClient.getInstance.groupManager.addGroupManagerListener(this);
    super.initState();
  }

  @override
  void dispose() {
    // 移除群组
    EMClient.getInstance.groupManager.removeGroupManagerListener(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container();
  }

  @override
  // 群组新增管理员
  void onAdminAddedFromGroup(
    String groupId,
    String admin,
  ) {}

  @override
  // 群组管理员被移除权限
  void onAdminRemovedFromGroup(
    String groupId,
    String admin,
  ) {}

  @override
  // 群组成员全员禁言状态变化
  void onAllGroupMemberMuteStateChanged(
    String groupId,
    bool isAllMuted,
  ) {}

  @override
  // 群组公告更新
  void onAnnouncementChangedFromGroup(
    String groupId,
    String announcement,
  ) {}

  @override
  // 用户自动同意进群邀请
  void onAutoAcceptInvitationFromGroup(
    String groupId,
    String inviter,
    String? inviteMessage,
  ) {}

  @override
  // 群组解散
  void onGroupDestroyed(
    String groupId,
    String? groupName,
  ) {}

  @override
  // 用户同意进群邀请
  void onInvitationAcceptedFromGroup(
    String groupId,
    String invitee,
    String? reason,
  ) {}

  @override
  // 用户拒绝进群邀请
  void onInvitationDeclinedFromGroup(
    String groupId,
    String invitee,
    String? reason,
  ) {}

  @override
  // 用户收到进群邀请
  void onInvitationReceivedFromGroup(
    String groupId,
    String? groupName,
    String inviter,
    String? reason,
  ) {}

  @override
  // 有群成员离开群组
  void onMemberExitedFromGroup(
    String groupId,
    String member,
  ) {}

  @override
  // 有新成员加入群组
  void onMemberJoinedFromGroup(
    String groupId,
    String member,
  ) {}

  @override
  // 有成员被添加到禁言列表
  void onMuteListAddedFromGroup(
    String groupId,
    List<String> mutes,
    int? muteExpire,
  ) {}

  @override
  // 有成员被移出禁言列表
  void onMuteListRemovedFromGroup(
    String groupId,
    List<String> mutes,
  ) {}

  @override
  // 群主变更
  void onOwnerChangedFromGroup(
    String groupId,
    String newOwner,
    String oldOwner,
  ) {}

  @override
  // 群主或群管理员同意用户的进群申请
  void onRequestToJoinAcceptedFromGroup(
    String groupId,
    String? groupName,
    String accepter,
  ) {}

  @override
  // 群主或群管理员拒绝用户的进群申请
  void onRequestToJoinDeclinedFromGroup(
    String groupId,
    String? groupName,
    String decliner,
    String? reason,
  ) {}

  @override
  // 群主或群管理员收到进群申请
  void onRequestToJoinReceivedFromGroup(
    String groupId,
    String? groupName,
    String applicant,
    String? reason,
  ) {}

  @override
  // 新增群组共享文件
  void onSharedFileAddedFromGroup(
    String groupId,
    EMGroupSharedFile sharedFile,
  ) {}

  @override
  // 删除群组共享文件
  void onSharedFileDeletedFromGroup(
    String groupId,
    String fileId,
  ) {}

  @override
  // 有成员被移出群组
  void onUserRemovedFromGroup(
    String groupId,
    String? groupName,
  ) {}

  @override
  // 有成员加入白名单
  void onAllowListAddedFromGroup(
    String groupId,
    List<String> members,
  ) {}

  @override
  // 有成员被移出白名单
  void onAllowListRemovedFromGroup(
    String groupId,
    List<String> members,
  ) {}
}
```