# 管理聊天室成员

聊天室是支持多人沟通的即时通讯系统。本文指导你如何使用环信即时通讯 IM Flutter SDK 在实时互动 app 中实现聊天室的相关功能。

## 技术原理

环信即时通讯 IM Flutter SDK 提供 `EMChatRoom`、`EMRoomManager` 和 `EMChatRoomManagerListener` 类用于聊天室管理，支持你通过调用 API 在项目中实现如下功能：

- 将成员移除聊天室
- 获取聊天室成员列表
- 管理聊天室黑名单
- 管理聊天室白名单
- 管理聊天室禁言列表
- 开启和关闭群组全员禁言
- 管理聊天室所有者及管理员

## 前提条件

开始前，请确保满足以下条件：

- 完成 SDK 初始化，详见 [快速开始](https://docs-im.easemob.com/ccim/flutter/quickstart)；
- 了解环信即时通讯 IM API 的接口调用频率限制，详见 [使用限制](https://docs-im.easemob.com/ccim/limitation)；
- 了解聊天室的数量限制，详见 [套餐包详情](https://www.easemob.com/pricing/im)。

## 实现方法

本节介绍如何使用环信即时通讯 IM Flutter SDK 提供的 API 实现上述功能。

### 将成员移出聊天室

仅聊天室所有者和聊天室管理员可以调用 `EMChatRoomManager#removeChatRoomMembers` 方法将指定成员移出聊天室。被移出后，该成员收到 `EMChatRoomManagerListener#onRemovedFromChatRoom` 回调，其他成员收到 `EMChatRoomManagerListener#onMemberExitedFromChatRoom` 回调。被移出聊天室后，该用户还可以再次加入聊天室。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.removeChatRoomMembers(
    roomId,
    members,
  );
} on EMError catch (e) {
}
```

### 获取聊天室成员列表

所有聊天室成员均可以调用 `EMChatRoomManager#fetchChatRoomMembers` 方法获取当前聊天室的成员列表。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.fetchChatRoomMembers(
    roomId,
    pageSize: pageSize,
    cursor: cursor,
  );
} on EMError catch (e) {
}
```

### 管理聊天室黑名单

#### 将成员加入聊天室黑名单

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#blockChatRoomMembers` 方法将指定成员添加至黑名单。被加入黑名单后，该成员收到 `EMChatRoomManagerListener#onRemovedFromChatRoom` 回调，其他成员收到 `EMChatRoomManagerListener#onMemberExitedFromChatRoom` 回调。被加入黑名单后，该成员无法再收发聊天室消息并被移出聊天室，黑名单中的成员如想再次加入聊天室，聊天室所有者或管理员必须先将其移除黑名单。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.blockChatRoomMembers(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 将成员移出聊天室黑名单

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#unBlockChatRoomMembers` 方法将成员移出聊天室黑名单。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.unBlockChatRoomMembers(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 获取聊天室黑名单

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#fetchChatRoomBlockList` 方法获取当前聊天室黑名单。

示例代码如下：

```dart
try {
  List<String>? list= await EMClient.getInstance.chatRoomManager.fetchChatRoomBlockList(
    roomId,
    pageNum: pageNum,
    pageSize: pageSize,
  );
} on EMError catch (e) {
}
```

### 管理聊天室白名单

#### 添加成员至聊天室白名单列表

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#addMembersToChatRoomAllowList` 方法将指定成员添加至聊天室白名单。被添加后，该成员和其他未操作的聊天室管理员或聊天室所有者收到 `EMChatRoomManagerListener#onAllowListAddedFromChatRoom` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.addMembersToChatRoomAllowList(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 将成员移出聊天室白名单列表

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#removeMembersFromChatRoomAllowList` 方法将成员移出聊天室禁言列表。被解除禁言后，该成员和其他未操作的聊天室管理员或聊天室所有者收到 `EMChatRoomManagerListener#onAllowListRemovedFromChatRoom` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.removeMembersFromChatRoomAllowList(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 检查自己是否在白名单中

所有群成员可以调用 `EMChatRoomManager#isMemberInChatRoomAllowList` 方法检查自己是否在群白名单中，示例代码如下：

示例代码如下：

```dart
try {
  bool isInAllowList = await EMClient.getInstance.chatRoomManager.isMemberInChatRoomAllowList(roomId);
} on EMError catch (e) {
}
```

### 管理聊天室禁言列表

#### 添加成员至聊天室禁言列表

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#muteChatRoomMembers` 方法将指定成员添加至聊天室禁言列表。被禁言后，该成员和其他未操作的聊天室管理员或聊天室所有者收到 `EMChatRoomManagerListener#onMuteListAddedFromChatRoom` 回调。

**注意**：
聊天室所有者可禁言聊天室所有成员，聊天室管理员可禁言聊天室普通成员。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.muteChatRoomMembers(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 将成员移出聊天室禁言列表

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#unMuteChatRoomMembers` 方法将成员移出聊天室禁言列表。被解除禁言后，该成员和其他未操作的聊天室管理员或聊天室所有者收到 `EMChatRoomManagerListener#onMuteListRemovedFromChatRoom` 回调。

**注意**
聊天室所有者可对聊天室所有成员解除禁言，聊天室管理员可对聊天室普通成员解除禁言。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.unMuteChatRoomMembers(
    roomId,
    members
  );
} on EMError catch (e) {
}
```

#### 获取聊天室禁言列表

仅聊天室所有者和管理员可以调用 `EMChatRoomManager#fetchChatRoomMuteList` 方法获取当前聊天室的禁言列表。

示例代码如下：

```dart
try {
  List<String>? list =
      await EMClient.getInstance.chatRoomManager.fetchChatRoomMuteList(
    roomId,
    pageNum: pageNum,
    pageSize: pageSize,
  );
} on EMError catch (e) {
}
```

### 开启和关闭群组全员禁言

为了快捷管理聊天室发言，拥有者和管理员可以开启和关闭聊天室全员禁言。全员禁言和单独的成员禁言不冲突，设置或者解除全员禁言，原禁言列表并不会变化。

#### 开启全员禁言

仅群主和群管理员可以调用 `EMChatRoomManager#muteAllChatRoomMembers` 方法开启全员禁言。全员禁言开启后，除了在白名单中的群成员，其他成员不能发言。调用成功后，聊天室成员会收到 `EMChatRoomManagerListener#onAllChatRoomMemberMuteStateChanged` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.muteAllChatRoomMembers();
} on EMError catch (e) {
}
```

#### 关闭全员禁言

仅群主和群管理员可以调用 `EMChatRoomManager#unMuteAllChatRoomMembers` 方法取消全员禁言。调用成功后，聊天室成员会收到 `EMChatRoomManagerListener#onAllChatRoomMemberMuteStateChanged` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.unMuteAllChatRoomMembers();
} on EMError catch (e) {
}
```

### 管理聊天室所有者和管理员

#### 变更聊天室所有者

仅聊天室所有者可以调用 `EMChatRoomManager#changeOwner` 方法将权限移交给聊天室中指定成员。成功移交后，原聊天室所有者变为聊天室成员，新的聊天室所有者和聊天室管理员收到 `EMChatRoomManagerListener#onOwnerChangedFromChatRoom` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.changeOwner(
    roomId,
    newOwner,
  );
} on EMError catch (e) {
}
```

#### 添加聊天室管理员

仅聊天室所有者可以调用 `EMChatRoomManager#addChatRoomAdmin` 方法添加聊天室管理员。成功添加后，新管理员及其他管理员收到 `EMChatRoomManagerListener#onAdminAddedFromChatRoom` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.addChatRoomAdmin(
    roomId,
    memberId,
  );
} on EMError catch (e) {
}
```

#### 移除聊天室管理员

仅聊天室所有者可以调用 `EMChatRoomManager#removeChatRoomAdmin` 方法移除聊天室管理员。成功移除后，被移除的管理员及其他管理员收到 `EMChatRoomManagerListener#onAdminRemovedFromChatRoom` 回调。

示例代码如下：

```dart
try {
  await EMClient.getInstance.chatRoomManager.removeChatRoomAdmin(
    roomId,
    adminId,
  );
} on EMError catch (e) {
}
```

### 监听聊天室事件

详见 [监听聊天室事件](添加链接)。